/******************************************************************************
 tomsamp:
 	Example using the Text Object Model (TOM)

	You need the riched20.dll
 	Link with oleaut32.lib for SysAllocString() and SysFreeString()

	Note: The formatting actions could have been done with regular
		  rich edit commands. I chose to change the font,
		  tabstop, and alignment because their result could be easily
		  seen. The more interesting actions are the ones from ITextPara.
*******************************************************************************/
#include <windows.h>
#include <tom.h>
#include <richole.h>
#include <richedit.h>
#include <stdio.h>
#include "TomSamp.h"

#define DEFINE_GUIDXXX(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
        EXTERN_C const GUID CDECL name \
                = { l, w1, w2, { b1, b2,  b3,  b4,  b5,  b6,  b7,  b8 } }

/* Set the id of the ITextDocument interface. */
DEFINE_GUIDXXX(IID_ITextDocument, 0x8CC497C0, 0xA1DF, 0x11CE, 0x80, 0x98,
                0x00, 0xAA, 0x00, 0x47, 0xBE, 0x5D);

/* Id for the rich edit control. */
#define ID_RICHEDWND 1

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);
void ShowErrMsg(HWND hwnd, DWORD dwError);
HRESULT ChangeTheFont(ITextRange *pTextRange);

static HANDLE hRtfLib;
static HINSTANCE hInst;
char szAppName[] = "TOMSample";

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
	HWND        hwnd ;
	MSG         msg ;
	WNDCLASSEX  wndclass ;

	hInst = hInstance;

	if(NULL == (hRtfLib = LoadLibrary("RICHED20.DLL"))){
		MessageBox(NULL, "Unable to Load Riched20.dll",
			NULL, MB_OK | MB_ICONEXCLAMATION);
		return 0;
	}

	wndclass.cbSize        = sizeof (wndclass) ;
	wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
	wndclass.lpfnWndProc   = WndProc ;
	wndclass.cbClsExtra    = 0 ;
	wndclass.cbWndExtra    = 0 ;
	wndclass.hInstance     = hInstance ;
	wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
	wndclass.lpszMenuName  = MAKEINTRESOURCE(MAIN_MENU) ;
	wndclass.lpszClassName = szAppName ;
	wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;

	RegisterClassEx (&wndclass) ;

	hwnd = CreateWindow (szAppName, szAppName, WS_OVERLAPPEDWINDOW,
			CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
			CW_USEDEFAULT, NULL, NULL, hInstance, NULL) ;

	ShowWindow (hwnd, iCmdShow) ;
	UpdateWindow (hwnd) ;

	while (GetMessage (&msg, NULL, 0, 0))
		{
			TranslateMessage (&msg) ;
			DispatchMessage (&msg) ;
		}
		return msg.wParam ;
}

LRESULT CALLBACK WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	static HWND hwndEdit;
	HRESULT hResult;
	static LPRICHEDITOLE pRichEdOle = NULL; // Pointer to an IRichEditOle object.
	static ITextDocument *pTextDoc = NULL;
	ITextRange *pTextRange = NULL;
	ITextPara *pTextPara = NULL;

	switch (iMsg)
		{
			case WM_CREATE :
				hwndEdit = CreateWindowEx (WS_EX_CLIENTEDGE, RICHEDIT_CLASS, NULL,
                         		WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL | WS_CLIPSIBLINGS |
                              		WS_BORDER | ES_LEFT | ES_MULTILINE |
                              		ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
                         		0, 0, 0, 0,
                         		hwnd, (HMENU) ID_RICHEDWND, hInst, NULL) ;

                /* Get a rich edit ole object. */
                if(!SendMessage(
  						hwndEdit,        		// handle to destination window
  						EM_GETOLEINTERFACE,		// message to send
  						0,        				// not used; must be zero
  						(LPARAM) &pRichEdOle   	// IRichEditOle object (LPVOID *)
						))
				{	/* Exit if error. */
					MessageBox(hwnd, "Error getting IRichEditOle object", szAppName, 0);
					SendMessage(hwnd, WM_CLOSE, 0, 0);
				}

				/* Exit if error. */
				hResult = pRichEdOle->lpVtbl->QueryInterface(pRichEdOle, &IID_ITextDocument,
															(LPVOID *)&pTextDoc);
				if(FAILED(hResult)){
					ShowErrMsg(hwnd, hResult);
					SendMessage(hwnd, WM_CLOSE, 0, 0);
				}

				return 0 ;

			case WM_COMMAND:

				switch(LOWORD(wParam)){
					case IDM_TABSTOPS:
						/* Change the tabstop.
						 * The default tab stops is 32, we'll cut it in half.
						 */
						hResult = pTextDoc->lpVtbl->SetDefaultTabStop(pTextDoc, 16);
						if(FAILED(hResult)){
							ShowErrMsg(hwnd, hResult);
						}
						break;

					case IDM_CHANGEFONT:

						/* Get an ITextRange object, and set the range
						 * to the whole document
						 */
						hResult = pTextDoc->lpVtbl->Range(pTextDoc, 0, 0xffff, &pTextRange);

						if(S_OK != hResult){
							ShowErrMsg(hwnd, hResult);
						}
						else{
							hResult = ChangeTheFont(pTextRange);
							if(FAILED(hResult)){
								ShowErrMsg(hwnd, hResult);
							}
						}
						if(pTextRange){
							pTextRange->lpVtbl->Release(pTextRange);
							pTextRange = NULL;
						}
						break;

					case IDM_ALIGNCENTER:
						/* Get an ITextRange object, and set the range
						 * to the whole document
						 */
						hResult = pTextDoc->lpVtbl->Range(pTextDoc, 0, 0xffff, &pTextRange);

						if(S_OK != hResult){
							ShowErrMsg(hwnd, hResult);
						}
						else{
							/* Get an ITextPara object */
							hResult = pTextRange->lpVtbl->GetPara(pTextRange, &pTextPara);
							if(FAILED(hResult)){
								ShowErrMsg(hwnd, hResult);
							}
							else{
								pTextPara->lpVtbl->SetAlignment(pTextPara, tomAlignCenter);
							}
						}
						if(pTextPara){
							pTextPara->lpVtbl->Release(pTextPara);
							pTextPara = NULL;
						}
						if(pTextRange){
							pTextRange->lpVtbl->Release(pTextRange);
							pTextRange = NULL;
						}
						break;

					default:
						break;
				}

				return 0;

			case WM_SETFOCUS:
				SetFocus(hwndEdit);
				return 0;

			case WM_SIZE:
				MoveWindow(hwndEdit, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
				return 0;

			case WM_CLOSE:
				DestroyWindow (hwnd);

				if(pTextDoc){
					pTextDoc->lpVtbl->Release(pTextDoc);
				}

				if(pRichEdOle){
					pRichEdOle->lpVtbl->Release(pRichEdOle);
				}

				if(hRtfLib){
					FreeLibrary(hRtfLib);
				}
				return 0;

			case WM_DESTROY :
				PostQuitMessage (0) ;
				return 0 ;
		}

	return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}

void ShowErrMsg(HWND hwnd, DWORD dwError)
{
	char buff[256];
	char buff2[20];

	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwError, 0, buff, 255, NULL);

	sprintf(buff2, "ERROR: %d", dwError);
	MessageBox(hwnd, buff, buff2, 0);
}

HRESULT ChangeTheFont(ITextRange *pTextRange)
{
	ITextFont *pTextFont;
	HRESULT hr ;
	OLECHAR fontname[20];

	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, "Arial", sizeof("Arial"),
						fontname, sizeof(fontname));

	BSTR bstrTemp = SysAllocString((BSTR)fontname);

	if(bstrTemp == NULL){
		return E_UNEXPECTED;
	}

	/* Get a font object. */
	hr = pTextRange->lpVtbl->GetFont(pTextRange, &pTextFont);
	if(FAILED(hr)){
		SysFreeString(bstrTemp);
		return hr;
	}
	pTextFont->lpVtbl->SetName(pTextFont, bstrTemp);
	pTextFont->lpVtbl->SetBold(pTextFont, tomFalse); /* Make the text not bold */
	pTextRange->lpVtbl->SetFont(pTextRange, pTextFont);
	pTextFont->lpVtbl->Release(pTextFont);
	SysFreeString(bstrTemp);
	return hr;
}
